/*
 * 该部分服务属于公共服务。
 * 该部分服务是一些自定的基础服务。
 * 主要用于对内置服务的一些快捷调用。
 */

// 跨域服务
angular.module("starter.services").factory("crossSiteService", [
  "$http",
  function ($http) {
    var debug = true;
    var http_config = {
      timeout: 15000,
      headers: {
        Authorization: window.localStorage.getItem("access_token"),
      },
    };

    function handlePromise(promise, callback) {
      if (!promise || !callback) {
        return;
      }
      promise
        .success(function (data, status, config, headers) {
          if (debug) {
            console.log(
              "debug info, promise.success:\n" +
                "data = " +
                data +
                "\n" +
                "status = " +
                status +
                "\n" +
                "config = " +
                config +
                "\n" +
                "headers = " +
                headers
            );
          }
          if (callback.onSuccess) {
            callback.onSuccess(data, status, config, headers);
          }
        })
        .error(function (data, status, headers, config) {
          if (debug) {
            console.log(
              "debug info, promise.error:\n" +
                "data = " +
                data +
                "\n" +
                "status = " +
                status +
                "\n" +
                "config = " +
                config +
                "\n" +
                "headers = " +
                headers
            );
          }
          //处理失败后的响应
          if (callback.onError) {
            callback.onError(data, status, headers, config);
          }
        })
        .finally(function () {
          if (callback.onFinally) {
            callback.onFinally();
          }
          if (callback.onAfter) {
            callback.onAfter();
          }
        });
    }

    // 使用get实现跨域请求
    function get(url, callback) {
      //
      if (!url || !callback) {
        return;
      }
      //
      if (callback.onBefore) {
        callback.onBefore();
      }
      //
      console.log("get. url = " + url);
      http_config.headers.Authorization = window.localStorage.getItem(
        "access_token"
      );
      var promise = $http.get(url, http_config);
      handlePromise(promise, callback);
    }
    // 使用jsonp实现跨域请求
    function jsonp(url, callback) {
      //
      if (!url || !callback) {
        return;
      }
      //
      if (callback.onBefore) {
        callback.onBefore();
      }
      //
      if (url.indexOf("callback=JSON_CALLBACK") < 0) {
        var questionMarkIndex = url.indexOf("?");
        if (questionMarkIndex < 0) {
          url = url + "?callback=JSON_CALLBACK";
        } else {
          if (questionMarkIndex < url.length - 1) {
            url = url + "&";
          }
          url = url + "callback=JSON_CALLBACK";
        }
      }
      console.log("jsonp. url = " + url);
      var promise = $http.jsonp(url, http_config);
      handlePromise(promise, callback);
    }

    // 通过$http.post实现跨域请求
    function post(url, data, callback) {
      //
      if (!url || !callback) {
        return;
      }
      //
      if (callback.onBefore) {
        callback.onBefore();
      }
      //
      console.log("post. url = " + url);
      http_config.headers.Authorization = window.localStorage.getItem(
        "access_token"
      );
      var promise = $http.post(url, data, http_config);
      handlePromise(promise, callback);
    }
    // 通过$http.post实现跨域请求
    function postForm(url, data, callback) {
      //
      if (!url || !callback) {
        return;
      }
      //
      if (callback.onBefore) {
        callback.onBefore();
      }

      var config = {
        headers: { "Content-Type": "application/x-www-form-urlencoded" },
        transformRequest: function (obj) {
          var str = [];
          for (var p in obj) {
            str.push(encodeURIComponent(p) + "=" + encodeURIComponent(obj[p]));
          }
          return str.join("&");
        },
      };
      var promise = $http.post(url, data, config);
      handlePromise(promise, callback);
    }
    // 通过$http.delete实现跨域请求
    function delete0(url, data, callback) {
      //
      if (!url || !callback) {
        return;
      }
      //
      if (callback.onBefore) {
        callback.onBefore();
      }
      //
      console.log("delete. url = " + url);
      http_config.headers.Authorization = window.localStorage.getItem(
        "access_token"
      );
      var promise = $http.delete(url, data, http_config);
      handlePromise(promise, callback);
    }
    // 通过$http.patch实现跨域请求
    function patch(url, data, callback) {
      //
      if (!url || !callback) {
        return;
      }
      //
      if (callback.onBefore) {
        callback.onBefore();
      }
      //
      console.log("patch. url = " + url);
      http_config.headers.Authorization = window.localStorage.getItem(
        "access_token"
      );
      var promise = $http.patch(url, data, http_config);
      handlePromise(promise, callback);
    }
    // 通过$http.put实现跨域请求
    function put(url, data, callback) {
      //
      if (!url || !callback) {
        return;
      }
      //
      if (callback.onBefore) {
        callback.onBefore();
      }
      //
      console.log("put. url = " + url);
      http_config.headers.Authorization = window.localStorage.getItem(
        "access_token"
      );
      var promise = $http.put(url, data, http_config);
      handlePromise(promise, callback);
    }

    return {
      get: get,
      jsonp: jsonp,
      post: post,
      postForm: postForm,
      patch: patch,
      delete0: delete0,
      put: put,
    };
  },
]);

// 图书服务
angular.module("starter.services").factory("bookService", [
  function () {
    // 修正缩略图缺少的问题
    function fixBookThumbnail(book) {
      if (book) {
        // 如果没有缩略图，使用默认图片代替。图片选择规则：medium --> large/small --> default
        var defaultBookThumbnailUrl = "img/book_thumbnail.jpg"; // 默认缩略图
        if (!book.image) {
          book.image = {
            large: defaultBookThumbnailUrl,
            medium: defaultBookThumbnailUrl,
            small: defaultBookThumbnailUrl,
          };
        } else {
          if (!book.image.large) {
            book.imageUrl.large =
              book.image.medium | book.image.small | defaultBookThumbnailUrl;
          }
          if (!book.image.medium) {
            book.image.medium =
              book.image.small | book.imageUrl.large | defaultBookThumbnailUrl;
          }
          if (!book.image.small) {
            book.image.small =
              book.image.medium | book.imageUrl.large | defaultBookThumbnailUrl;
          }
        }
      }
    }
    //
    function fixBook(book) {
      fixBookThumbnail(book);
    }
    //
    function fixBooks(books) {
      if (books) {
        for (var i = 0; i < books.length; i++) {
          fixBook(books[i]);
        }
      }
    }

    // 图书分享
    function getBookSharingUrl(isbn) {
      return baseServerUrl + "/book/share/" + isbn;
    }

    //
    return {
      fixBookThumbnail: fixBookThumbnail,
      fixBook: fixBook,
      fixBooks: fixBooks,
      getBookSharingUrl: getBookSharingUrl,
    };
  },
]);
