angular
  .module("starter.controllers")

  .controller(
    "MineAvatorCtrl",
    function (
      $rootScope,
      $scope,
      $ionicTabsDelegate,
      $ionicActionSheet,
      $ionicLoading,
      apiService,
      uiService,
      ngCordovaService,
      $ionicHistory,
      $cordovaImagePicker,
      $cordovaCamera,
      $cordovaFileTransfer,
      $ionicTabsDelegate
    ) {
      //
      $ionicTabsDelegate.showBar(false);

      // ---------------------------------------------------------------------------------------------
      if (!$rootScope.userInfo) {
        // 获取用户信息
        apiService.getUserInfo($rootScope.user.account, {
          onSuccess: function (data, status, config, headers) {
            // 保存用户信息
            $rootScope.userInfo = data;
          },
        });
      }

      $scope.data = {
        status: null,
        newAvatorLocalUrl: null,
      };
      $scope.cancelNewAvator = function () {
        $scope.data.status = "show_avator";
      };
      $scope.showMenu = function () {
        if ($scope.data.status == "show_avator") {
          $scope.showSelectPictureMenu();
        } else if ($scope.data.status == "confirm_avator") {
          $scope.showConfirmPictureMenu();
        }
      };
      //
      $scope.showSelectPictureMenu = function () {
        $ionicActionSheet.show({
          buttons: [{ text: "从相册选择图片" }, { text: "拍照" }],
          titleText: "更新头像",
          cancelText: "取消",
          buttonClicked: function (index) {
            switch (index) {
              case 0:
                $scope.pickImageFromLocal();
                break;
              case 1:
                $scope.capturePictureByCamera();
                break;
              default:
                break;
            }
            return true;
          },
        });
      };
      // 显示确认图片按钮
      $scope.showConfirmPictureMenu = function () {
        $ionicActionSheet.show({
          buttons: [{ text: "上传头像" }, { text: "原始头像" }],
          titleText: "更新头像",
          cancelText: "取消",
          buttonClicked: function (index) {
            switch (index) {
              case 0:
                $scope.uploadNewAvator();
                break;
              case 1:
                $scope.data.status = "show_avator";
                break;
              default:
                break;
            }
            return true;
          },
        });
      };

      // ---------------------------------------------------------------------------------------------
      $scope.pickImageFromLocal = function () {
        var options = {
          maximumImagesCount: 1,
          // width: 800,
          // height: 800,
          // quality: 80
        };
        $cordovaImagePicker.getPictures(options).then(
          function (results) {
            for (var i = 0; i < results.length; i++) {
              console.log("pickImageFromLocal. Image URI: " + results[i]);
            }
            if (results.length == 1) {
              $scope.data.newAvatorLocalUrl = results[0];
              $scope.data.status = "confirm_avator";
            }
          },
          function (error) {
            // error getting photos
          }
        );
      };

      // ---------------------------------------------------------------------------------------------
      $scope.capturePictureByCamera = function () {
        var options = {
          destinationType: Camera.DestinationType.FILE_URI,
          sourceType: Camera.PictureSourceType.CAMERA,
          correctOrientation: true, //ios上所拍的图片方向不对
          saveToPhotoAlbum: true,
        };

        // var options = {
        //     //这些参数可能要配合着使用，比如选择了sourcetype是0，destinationtype要相应的设置
        //     quality: 100,                                            //相片质量0-100
        //     destinationType: Camera.DestinationType.FILE_URI,        //返回类型：DATA_URL= 0，返回作为 base64 編碼字串。 FILE_URI=1，返回影像档的 URI。NATIVE_URI=2，返回图像本机URI (例如，資產庫)
        //     sourceType: Camera.PictureSourceType.CAMERA,             //从哪里选择图片：PHOTOLIBRARY=0，相机拍照=1，SAVEDPHOTOALBUM=2。0和1其实都是本地图库
        //     allowEdit: false,                                        //在选择之前允许修改截图
        //     encodingType: Camera.EncodingType.JPEG,                   //保存的图片格式： JPEG = 0, PNG = 1
        //     // targetWidth: 200,                                        //照片宽度
        //     // targetHeight: 200,                                       //照片高度
        //     mediaType: 0,                                             //可选媒体类型：圖片=0，只允许选择图片將返回指定DestinationType的参数。 視頻格式=1，允许选择视频，最终返回 FILE_URI。ALLMEDIA= 2，允许所有媒体类型的选择。
        //     cameraDirection: 0,                                       //枪后摄像头类型：Back= 0,Front-facing = 1
        //     popoverOptions: CameraPopoverOptions,
        //     saveToPhotoAlbum: true                                   //保存进手机相册
        // };

        $cordovaCamera.getPicture(options).then(
          function (imageURI) {
            console.log("capturePictureByCamera. Image URI: " + imageURI);
            $scope.data.newAvatorLocalUrl = imageURI;
            $scope.data.status = "confirm_avator";
          },
          function (err) {
            // error
            console.log("capturePictureByCamera. Image URI err : " + err);
          }
        );
        // $cordovaCamera.cleanup(); // only for FILE_URI
      };

      // ---------------------------------------------------------------------------------------------
      // 文件上传
      $scope.uploadNewAvator = function () {
        uiService.ionicLoadingShow({ templateTitle: "正在上传头像..." });
        var server =
          baseServerUrl + "/news/people_image_upload/" + $scope.user.account;
        var filePath = $scope.data.newAvatorLocalUrl;
        var options = null;
        $cordovaFileTransfer.upload(server, filePath, options).then(
          function (result) {
            // Success!
            $ionicLoading.hide();
            console.log("已经成功地更新了您的头像");
            ngCordovaService.showToast("已经成功地更新了您的头像");
            $rootScope.userInfo.photo_url = result.response; // 更新全局scope
            // TODO 给更新url加上一个不存在的随机参数，让浏览器以为这是一个新的url。目的是，防止原有的头像url和新的头像url相同的时候，浏览器认为该url已经做了缓存，不再重新加载图片。
            var urlAppend = "";
            if (
              $rootScope.userInfo.photo_url.indexOf("?") ==
              $rootScope.userInfo.photo_url.length
            ) {
              urlAppend = new Date().getTime();
            } else {
              urlAppend = "?v=" + new Date().getTime();
            }
            $rootScope.userInfo.photo_url =
              $rootScope.userInfo.photo_url + urlAppend;
            //
            $scope.data.status = "show_avator";
          },
          function (err) {
            // Error
            $ionicLoading.hide();
            ngCordovaService.showToast("很遗憾您的头像没有更新成功");
            console.warn("很遗憾您的头像没有更新成功");
            console.log("Failed file write: " + err.toString());
          },
          function (progress) {
            var p = progress.loaded / progress.total;
            p = Math.floor(p * 100) + "%";
            console.log("progress = " + p);
          }
        );
      };

      // ---------------------------------------------------------------------------------------------
      //
      $scope.data.status = "show_avator";
      $ionicTabsDelegate.showBar(false);
    }
  );
